/***************************************************************************
 *   Copyright (C) 2002-2006, the RPPPPoEK developers                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "rppppoek.h"

#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kstdaction.h>

#include <qfile.h>
#include <qtooltip.h>
#include <qclipboard.h>

rppppoek::rppppoek(KDialog* dialog, QWidget *parent, const char *name) : KSystemTray(parent, name),  dlgAbout(dialog)
{
	KActionCollection* actionCollection = new KActionCollection(this);

	prConnect = new KProcess;
	*prConnect << SCRIPT << "connect";

	prDisconnect = new KProcess;
	*prDisconnect << SCRIPT << "disconnect";

	prStatus = new KProcess;
	*prStatus << SCRIPT << "status" << DEVICE_NAME;

	sAppName			= caption();
	sConnecting			= i18n("Connecting");
	sDisconnecting		= i18n("Disconnecting");
	sConnected			= i18n("Connected");
	sDisconnected		= i18n("Not connected");
	sConnect			= i18n("Connect");
	sDisconnect			= i18n("Disconnect");

	pixConnecting		= loadIcon("connecting");
	pixConnected		= loadIcon("connected");
	pixDisconnected		= loadIcon("disconnected");
	for (int i = 0; i < TIMER_ANIMATION_COUNT; ++i)
		pixTraffic[i]	= loadIcon(QString("traffic%1").arg(i));

	KAction* acAbout	= KStdAction::aboutApp(this, SLOT(slotAbout()), actionCollection);
	acConnect			= new KAction(sConnect,					loadIcon("connect_established"),	0, 0, 0, actionCollection, 0);
	acDisconnect		= new KAction(sDisconnect,				loadIcon("connect_no"),				0, 0, 0, actionCollection, 0);
	acShowIpAddress		= new KAction(i18n("Show IP address"),	loadIcon("info"),					0, 0, 0, actionCollection, 0);

	connect(acConnect,			SIGNAL(activated()),				this, SLOT(slotConnect()));
	connect(acDisconnect,		SIGNAL(activated()),				this, SLOT(slotDisconnect()));
	connect(acShowIpAddress,	SIGNAL(activated()),				this, SLOT(slotDisplayIpAddress()));
	connect(prConnect,			SIGNAL(processExited(KProcess*)),	this, SLOT(slotConnected()));
	connect(prDisconnect,		SIGNAL(processExited(KProcess*)),	this, SLOT(slotDisconnected()));
	connect(prStatus,			SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotConsumeStatus(KProcess*, char*, int)));

	iRXprev = iTXprev = -1;
	bAnimateTrayIcon = false;
	iCurrentTrayIcon = -1;

	bConnecting = false;
	bConnected = getTraffic();

	setIcon();
	setToolTip();
	setMenu();

	iconCheckTimer = new QTimer(this);
	connect(iconCheckTimer,		SIGNAL(timeout()),					this, SLOT(slotCheckTraffic()));

	iconAnimationTimer = new QTimer(this);
	connect(iconAnimationTimer,	SIGNAL(timeout()),					this, SLOT(slotDisplayTraffic()));

	iconCheckTimer->start(TIMER_CHECK_INTERVAL, true); // check for traffic; single shot, will be restarted in timeout() proc
	iconAnimationTimer->start(TIMER_ANIMATION_INTERVAL, true); // refresh tray icon; single shot as well

	contextMenu()->clear();
	contextMenu()->insertTitle(loadIcon("kppp"), i18n("Roaring Penguin PPPoE Kontrol"), 0);
	acConnect->plug(contextMenu(), 1);
	acDisconnect->plug(contextMenu(), 2);
	contextMenu()->insertSeparator(3);
	acShowIpAddress->plug(contextMenu(), 4);
	contextMenu()->insertSeparator(5);
	acAbout->plug(contextMenu(), 6);
}

rppppoek::~rppppoek()
{
}

bool rppppoek::getTraffic()
{
	bool bFoundDevice = false;

	QFile file(DEVICE_LIST);
	if (file.open(IO_ReadOnly)) {
		QTextStream stream(&file);
		while (!stream.atEnd()) {
			QString line = stream.readLine();
			QString lineSimplified = line.simplifyWhiteSpace();
			QString device = lineSimplified.section(':', 0, 0);

			if (device == DEVICE_NAME) {
				QString lineRest = lineSimplified.section(':', 1).simplifyWhiteSpace();
				QString rxBytes = lineRest.section(' ', 0, 0);
				QString txBytes = lineRest.section(' ', 8, 8);

				iRX = rxBytes.toInt();
				iTX = txBytes.toInt();

				bFoundDevice = true;
				break;
			}
		}
	}
	return bFoundDevice;
}

void rppppoek::setIcon()
{
	if (bConnecting)
		setPixmap(pixConnecting);

	else if (!bConnected)
		setPixmap(pixDisconnected);

	else if (!bAnimateTrayIcon)
		setPixmap(pixConnected);

	else
		setPixmap(pixTraffic[iCurrentTrayIcon]);
}

void rppppoek::setToolTip()
{
	QToolTip::remove(this);
	QString sPrefix = sAppName + " - ";

	if (bConnecting)
	{
		if (bConnected) // I'm disconnecting
			QToolTip::add(this, sPrefix + sDisconnecting);
		else
			QToolTip::add(this, sPrefix + sConnecting);
	}

	else if (!bConnected)
		QToolTip::add(this, sPrefix + sDisconnected);

	else // no separate text for active traffic
		QToolTip::add(this, sPrefix + sConnected);
}

void rppppoek::setMenu()
{
	acConnect->setEnabled(!bConnected && !bConnecting);

	acDisconnect->setEnabled(bConnected && !bConnecting);
	acShowIpAddress->setEnabled(bConnected && !bConnecting);
}

/** Connect to Internet */
void rppppoek::slotConnect()
{
	bConnecting = true;
	setIcon();
	setToolTip();
	setMenu();
	acConnect->setText(sConnect + " (" + i18n("Trying to connect") + ")");
	prConnect->start(KProcess::NotifyOnExit);
}

/** Called when start terminates */
void rppppoek::slotConnected()
{
	bConnecting = false;
	if (prConnect->normalExit() && !prConnect->exitStatus())
		bConnected = true;
	else
		KMessageBox::sorry(this, i18n("Couldn't connect"));

	setIcon();
	setToolTip();
	setMenu();
	acConnect->setText(sConnect);
}

/** Disconnect from Internet */
void rppppoek::slotDisconnect()
{
	bConnecting = true;	// technically speaking this is wrong, but in case the disconnect takes a long time
	setIcon();			// (for whatever reason), provide a visual feedback
	setToolTip();
	setMenu();
	acDisconnect->setText(sDisconnect + " (" + i18n("Trying to disconnect") + ")");
	prDisconnect->start(KProcess::NotifyOnExit);
}

/** Called when stop terminates */
void rppppoek::slotDisconnected()
{
	bConnecting = false;
	if (prDisconnect->normalExit() && !prDisconnect->exitStatus())
		bConnected = false;
	else
		KMessageBox::sorry(this, i18n("Couldn't disconnect"));

	setIcon();
	setToolTip();
	setMenu();
	acDisconnect->setText(sDisconnect);
}

/** Consume status (=IP address) from script */
void rppppoek::slotConsumeStatus(KProcess* /* proc */, char* buf, int /* len */)
{
	sIpAddress = buf;
	bIpAddressValid = true;
}

/** Display IP address */
void rppppoek::slotDisplayIpAddress()
{
	bIpAddressValid = false;
	prStatus->start(KProcess::Block, KProcess::Stdout);
	if (!prStatus->normalExit() || prStatus->exitStatus() || !bIpAddressValid)
		KMessageBox::error(this, i18n("Couldn't find an IP address.\nPlease make sure you're connected."));
	else
		if (KMessageBox::questionYesNo(this, i18n("Your IP address is\n%1\n\nDo you want to copy it to the clipboard?").arg(sIpAddress)) == KMessageBox::Yes)
			kapp->clipboard()->setText(sIpAddress);
}

/** Check for traffic */
void rppppoek::slotCheckTraffic()
{
	if (!bConnecting)
	{
		if (getTraffic())
		{
			if (!bConnected) // interface brought up from external?
			{
				bConnected = true;
				setIcon();
				setToolTip();
				setMenu();
			}

			if (iRXprev != -1) // previously got values?
				bAnimateTrayIcon = (iRXprev != iRX || iTXprev != iTX);
			iRXprev = iRX;
			iTXprev = iTX;
		}
		else
		{
			bAnimateTrayIcon = false;
			if (bConnected) // interface brought down from external?
			{
				bConnected = false;
				setIcon();
				setToolTip();
				setMenu();
			}
		}
	}

	iconCheckTimer->start(TIMER_CHECK_INTERVAL, true); // restart timer
}

/** Display traffic as tray icon animation */
void rppppoek::slotDisplayTraffic()
{
	if (bAnimateTrayIcon)
	{
		if (iCurrentTrayIcon == -1)
			setToolTip();
		iCurrentTrayIcon = (iCurrentTrayIcon + 1) % TIMER_ANIMATION_COUNT;
		setIcon();
	}
	else if (iCurrentTrayIcon != -1)
	{
		iCurrentTrayIcon = -1;
		setIcon();
		setToolTip();
	}
	iconAnimationTimer->start(TIMER_ANIMATION_INTERVAL, true); // restart timer
}

/** Show About dialog */
void rppppoek::slotAbout()
{
	dlgAbout->exec();
}

#include "rppppoek.moc"
